"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeServer = exports.ServerHttp2 = void 0;
const http2_1 = require("http2");
const utils_1 = require("../../lib/utils");
const crypto_1 = require("crypto");
const zlib_1 = require("zlib");
const already_1 = require("already");
const get_stream_1 = require("get-stream");
const server_common_1 = require("./server-common");
const { HTTP2_HEADER_PATH, HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_CONTENT_LENGTH, HTTP2_HEADER_ACCEPT_ENCODING, HTTP2_HEADER_SET_COOKIE, HTTP2_HEADER_LOCATION, } = http2_1.constants;
class ServerHttp2 extends server_common_1.TypedServer {
    constructor(opts) {
        super();
        this._awaits = [];
        this._opts = opts || {};
        if (this._opts.serverOptions)
            this._server = http2_1.createSecureServer(this._opts.serverOptions);
        else
            this._server = http2_1.createServer();
        this._sessions = new Set();
        this.port = null;
        this._server.on("stream", (stream, headers) => {
            const awaitStream = this.onStream(stream, headers)
                .catch(err => {
                console.error("Unit test server failed", err.stack);
                process.exit(1);
            })
                .then(() => {
                const index = this._awaits.findIndex(promise => promise === awaitStream);
                if (index !== -1)
                    this._awaits.splice(index, 1);
            });
            this._awaits.push(awaitStream);
        });
    }
    async _shutdown() {
        for (const session of this._sessions) {
            session.destroy();
        }
        await Promise.all(this._awaits);
        this._sessions.clear();
    }
    async onStream(stream, headers) {
        this._sessions.add(stream.session);
        stream.session.once("close", () => this._sessions.delete(stream.session));
        const path = headers[HTTP2_HEADER_PATH];
        let m;
        if (path === "/headers") {
            stream.respond({
                ":status": 200,
                "content-type": "application/json",
            });
            stream.end(JSON.stringify(headers));
        }
        else if (path === "/echo") {
            const responseHeaders = {
                ":status": 200,
            };
            [HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_CONTENT_LENGTH]
                .forEach(name => {
                responseHeaders[name] = headers[name];
            });
            stream.respond(responseHeaders);
            utils_1.pipeline(stream, stream);
        }
        else if (path === "/set-cookie") {
            const responseHeaders = {
                ":status": 200,
                [HTTP2_HEADER_SET_COOKIE]: [],
            };
            const data = await get_stream_1.buffer(stream);
            const json = JSON.parse(data.toString());
            json.forEach((cookie) => {
                responseHeaders[HTTP2_HEADER_SET_COOKIE]
                    .push(cookie);
            });
            stream.respond(responseHeaders);
            stream.end();
        }
        // tslint:disable-next-line
        else if (m = path.match(/\/wait\/(.+)/)) {
            const timeout = parseInt(m[1], 10);
            await already_1.delay(timeout);
            const responseHeaders = {
                ":status": 200,
            };
            [HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_CONTENT_LENGTH]
                .forEach(name => {
                responseHeaders[name] = headers[name];
            });
            try {
                stream.respond(responseHeaders);
                utils_1.pipeline(stream, stream);
            }
            catch (err) 
            // We ignore errors since this route is used to intentionally
            // timeout, which causes us to try to write to a closed stream.
            { }
        }
        else if (path === "/trailers") {
            const responseHeaders = {
                ":status": 200,
            };
            const data = await get_stream_1.buffer(stream);
            const json = JSON.parse(data.toString());
            stream.once("wantTrailers", () => {
                // TODO: Fix when @types/node is fixed
                stream.sendTrailers(json);
            });
            stream.respond(responseHeaders, 
            // TODO: Fix when @types/node is fixed
            {
                waitForTrailers: true,
            });
            stream.write("trailers will be sent");
            stream.end();
        }
        else if (path === "/sha256") {
            const hash = crypto_1.createHash("sha256");
            const responseHeaders = {
                ":status": 200,
            };
            stream.respond(responseHeaders);
            hash.on("readable", () => {
                const data = hash.read();
                if (data) {
                    stream.write(data.toString("hex"));
                    stream.end();
                }
            });
            utils_1.pipeline(stream, hash);
        }
        else if (path === "/push") {
            const responseHeaders = {
                ":status": 200,
            };
            const data = await get_stream_1.buffer(stream);
            const json = JSON.parse(data.toString());
            json.forEach((pushable) => {
                function cb(err, pushStream) {
                    if (err)
                        return;
                    if (pushable.data)
                        pushStream.write(pushable.data);
                    pushStream.end();
                }
                stream.pushStream(pushable.headers || {}, cb);
            });
            stream.respond(responseHeaders);
            stream.write("push-route");
            stream.end();
        }
        else if (path.startsWith("/compressed/")) {
            const encoding = path.replace("/compressed/", "");
            const accept = headers[HTTP2_HEADER_ACCEPT_ENCODING];
            if (!accept.includes(encoding)) {
                stream.destroy();
                return;
            }
            const encoder = encoding === "gzip"
                ? zlib_1.createGzip()
                : encoding === "deflate"
                    ? zlib_1.createDeflate()
                    : encoding === "br"
                        ? zlib_1.createBrotliCompress()
                        : null;
            const responseHeaders = {
                ":status": 200,
                "content-encoding": encoding,
            };
            stream.respond(responseHeaders);
            if (encoder)
                utils_1.pipeline(stream, encoder, stream);
            else
                utils_1.pipeline(stream, stream);
        }
        else if (path.startsWith("/goaway")) {
            const waitMs = path.startsWith("/goaway/")
                ? parseInt(path.replace("/goaway/", ""), 10)
                : 0;
            const responseHeaders = {
                ":status": 200,
                [HTTP2_HEADER_CONTENT_LENGTH]: "10",
            };
            stream.respond(responseHeaders);
            stream.write("abcde");
            stream.session.goaway();
            if (waitMs > 0)
                await already_1.delay(waitMs);
            server_common_1.ignoreError(() => stream.write("fghij"));
            server_common_1.ignoreError(() => stream.end());
        }
        else if (path.startsWith("/delay/")) {
            const waitMs = parseInt(path.replace("/delay/", ""), 10);
            if (waitMs > 0)
                await already_1.delay(waitMs);
            const responseHeaders = {
                ":status": 200,
                [HTTP2_HEADER_CONTENT_LENGTH]: "10",
            };
            server_common_1.ignoreError(() => stream.respond(responseHeaders));
            server_common_1.ignoreError(() => stream.write("abcde"));
            server_common_1.ignoreError(() => stream.write("fghij"));
            server_common_1.ignoreError(() => stream.end());
        }
        else if (path.startsWith("/slow/")) {
            const waitMs = parseInt(path.replace("/slow/", ""), 10);
            const responseHeaders = {
                ":status": 200,
                [HTTP2_HEADER_CONTENT_LENGTH]: "10",
            };
            stream.respond(responseHeaders);
            stream.write("abcde");
            if (waitMs > 0)
                await already_1.delay(waitMs);
            server_common_1.ignoreError(() => stream.write("fghij"));
            server_common_1.ignoreError(() => stream.end());
        }
        else if (path.startsWith("/prem-close")) {
            stream.close();
        }
        else if (path.startsWith("/redirect/")) {
            const redirectTo = path.slice(10).startsWith("http")
                ? path.slice(10)
                : path.slice(9);
            const responseHeaders = {
                ":status": 302,
                [HTTP2_HEADER_LOCATION]: redirectTo,
            };
            stream.respond(responseHeaders);
            stream.end();
        }
        else {
            const matched = (this._opts.matchers || [])
                .some(matcher => matcher({ path, stream, headers }));
            if (!matched) {
                stream.respond({ ":status": 400 });
                stream.end();
            }
        }
        if (!stream.closed)
            return new Promise(resolve => stream.once("close", resolve));
    }
}
exports.ServerHttp2 = ServerHttp2;
async function makeServer(opts = {}) {
    opts = opts || {};
    const server = new ServerHttp2(opts);
    await server.listen(opts.port);
    return { server, port: server.port };
}
exports.makeServer = makeServer;
//# sourceMappingURL=server-http2.js.map